close all; clear all;

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% PRESTAZIONI DEL MOTORE ALL'EQUILIBRIO  %%%

% verifico che all'equilibrio, con tensione di ingresso ueq pari a 1V
% e assenza di disturbo, y=1000 giri/min
 
R=48*10^(-3);
L=75*10^(-2);
J=8.37*10^(-7);
q=8.7*10^(-5);
h=9.1*10^(-3);
k=0.2*10^(-3);

A=[-R/L  -h/L
    h/J   -q/J];

bu=[1/L   0]';         % ruolo di u sullo stato
bv=[0  -k/J]';         % ruolo di d sullo stato

c=[0     60/(2*pi)];   % velocit albero motore y=x2*60/(2*pi)

d=0;

% equilibrio in assenza di disturbo
ueq=1;               % tensione di ingresso pari a 1V
xeq=-inv(A)*bu*ueq;
yeq=c*xeq+d*ueq;     % giri/min albero motore
yeq

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% PRESTAZIONI DEL MOTORE IN ANELLO APERTO  %%%%

% simulazione: da motore fermo a 1000 giri/min (w) in assenza di disturbo
[NUM,DEN]=ss2tf(A,bu,c,d);
G=tf(NUM,DEN);                 % fdt da w a y

% assenza di disturbo
T=linspace(0,6,1000);
w=1000;           % velocit desiderata
u=w/dcgain(G);    % y=G(0)u  -> u=w/G(0)
y_w=u*step(G,T);
figure(1);
plot(T,y_w); hold on; xlabel('tempo [s]'); ylabel('rpm [giri/min]'); title('ROTAZIONE [rpm] SENZA DISTURBO');

% pause;

% calcolo del tempo di risposta
autov=eig(A);
autov_dom=max(autov);
TD=-1/autov_dom;
TR=5*TD     % TR  circa pari a 3.7s (troppo!)

% presenza di disturbo costante: raffica di vento sull'elica da motore fermo a 1000 giri/min (w)
[NUMv,DENv]=ss2tf(A,bv,c,d);
Gv=tf(NUMv,DENv);              % fdt da v a y
v=100;
y_v=v*step(Gv,T);
ytot=y_w+y_v;
figure(1); plot(T,ytot); title('ROTAZIONE [rpm] CON DISTURBO');
% in anello aperto il disturbo "si ritrova" sull'uscita

% il valore di regime di y risente del disturbo v
y=dcgain(G)*u+dcgain(Gv)*v;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
%%% PRESTAZIONI DEL MOTORE CON CONTROLLORE PI  %%%%

mu=1;
C=mu*tf([1 2],[1 0]);
L=C*G;
figure(2); margin(L); grid;

[mod,fase]=bode(L,10)

mu=mu/mod;
C=mu*tf([1 2],[1 0]);
L=C*G;
figure(3); margin(L); grid;

% simulo senza disturbo
y_w=w*step(L/(1+L),T);
figure(4);
plot(T,y_w); hold on;
% simulo con disturbo
y_v=v*step(Gv/(1+L),T);
ytot=y_w+y_v;
figure(4); plot(T,ytot,'r');
xlabel('tempo [s]'); ylabel('rpm [giri/min]');
title('ROTAZIONE [rpm] CON (rosso) / SENZA (blu) DISTURBO');

% tempo di risposta con disturbo
roots(DENv)
